import { WidgetModel } from '../Widget/WidgetModel';
// import { HostProfile } from './hostProfile';
// import { UserContent } from './userContent';
// import { Outliers } from './outliers';
// import { Dns } from './dns';
import { Dashboard } from '../../types'

const MAX_DASHBOARD_WIDGETS = 10;

export const DASHBOARD_TYPE = {
  USER: 0,
  PROFILER: 1,
  DNS: 2,
  OUTLIERS: 3
};
const { USER, PROFILER, DNS, OUTLIERS } = DASHBOARD_TYPE;

export function getDashboardNameByType(type) {
  return {
    [USER]: 'Dashboard',
    [PROFILER]: 'Host Profiler',
    [DNS]: 'DNS Analysis',
    [OUTLIERS]: 'Outliers'
  }[type];
}

const asIdArray = (layout) => {
  const widgetIdArray = [];
  layout.forEach(({ front, back }) => widgetIdArray.push(front, back));
  return widgetIdArray.filter(Boolean);
};

const find = (dashboard, widget, mode = 0) => {
  const id = widget.getId();
  return dashboard.layout.find(({ front, back }) =>
    (mode >= 0 && front === id) || (mode <= 0 && back === id)
  );
};

export class DashboardModel implements Dashboard {
  id: string
  type: string | number
  name: string
  widgets: {}
  layout: any[]
  content: any
  disabled?: boolean

  constructor({
    id,
    type,
    name,
    widgets,
    layout,
    content,
    disabled // client-only
  }: Dashboard) {
    this.id = id;
    this.widgets = widgets || {};
    this.layout = layout || [];
    this.type = type || 0;
    this.name = name || '';
    this.content = content;
    this.disabled = disabled;
  }

  serialize() {
    return { ...this };
  }

  getId() {
    return this.id;
  }

  setId(id) {
    return new DashboardModel({ ...this, id });
  }

  getType() {
    return this.type;
  }

  getName() {
    return this.name;
  }

  setName(name) {
    return new DashboardModel({ ...this, name });
  }

  isFullView() {
    return this.getType() === OUTLIERS;
  }

  getWidgets() {
    const { widgets, layout } = this;
    return asIdArray(layout).map(id => new WidgetModel(widgets[id]));
  }

  getRefreshableWidgets(relativeOnly) {
    const widgets = this.getWidgets();
    return widgets /* (relativeOnly ? widgets.filter(widget => widget.hasRefresh()) : widgets)
      .filter(widget => widget.canRefresh());*/
  }

  removeWidget(widget: WidgetModel) {
    const id = widget.getId();
    const item = find(this, widget);
    if (item) {
      const widgets = { ...this.widgets };
      delete widgets[id];
      const newItem = item.front && item.back
        ? { front: item.front === id ? item.back : item.front }
        : undefined;
      const layout = newItem
        ? this.layout.map(anItem => (anItem === item ? newItem : anItem))
        : this.layout.filter(anItem => anItem !== item);
      return new DashboardModel({ ...this, layout, widgets });
    }
    return this;
  }

  updateWidget(widget) {
    const id = widget.getId();
    const widgets = { ...this.widgets, [id]: widget.serialize() };
    const item = find(this, widget);
    const layout = item ? this.layout : this.layout.concat([{ front: id }]);
    return new DashboardModel({ ...this, widgets, layout });
  }

  hasWidget(widget) {
    return !!this.getWidgetById(widget.getId());
  }

  getWidgetById(uuid) {
    const widgetObj = this.widgets[uuid];
    return widgetObj
      ? new WidgetModel({ ...widgetObj })
      : undefined;
  }

  getWidgetByContentId(uuid) {
    return this.getWidgets()[0]/*
      .find(widget => widget.getContent().getId() === uuid);*/
  }

  hasRefresh() {
    return true // this.getWidgets().some(widget => widget.hasRefresh());
  }

  canRefresh() {
    return true
    /*
    const widgets = this.getRefreshableWidgets();
    return (widgets && widgets.length > 0) ?
      widgets.every(widget => widget.canRefresh()) : false;*/
  }

  canAddWidget() {
    return asIdArray(this.layout).length < MAX_DASHBOARD_WIDGETS;
  }

  isEmpty() {
    return asIdArray(this.layout).length === 0;
  }

  setContent(contentObj) {
    const content = contentObj ? contentObj.serialize() : undefined;
    return new DashboardModel({ ...this, content });
  }

  getContent() {
    /*
    switch (this.getType()) {
      case PROFILER:
        return new HostProfile(this.content);
      case USER:
        return new UserContent(this.content);
      case DNS:
        return new Dns(this.content);
      case OUTLIERS:
        return new Outliers(this.content);
      default:
        return undefined;
    }*/
  }

  getFrontItems() {
    return this.layout
      .map(({ front }) => front)
      .filter(Boolean)
      .map(front => new WidgetModel(this.widgets[front]));
  }

  getBackItem(frontWidget) {
    const item = find(this, frontWidget, 1);
    return item && item.back ? new WidgetModel(this.widgets[item.back]) : undefined;
  }

  isOverlay(frontWidget) {
    const item = find(this, frontWidget, 1);
    return item && item.style === 'overlay';
  }

  isCollapsed(widget) {
    const item = find(this, widget);
    return item && item.collapsed;
  }

  setCollapsed(widget, collapsed) {
    const item = find(this, widget);
    if (item) {
      const layout = this.layout.map(anItem =>
        (anItem === item ? { ...item, collapsed } : anItem)
      );
      return new DashboardModel({ ...this, layout });
    }
    return this;
  }

  isDisabled() { // client-only
    return !!this.disabled;
  }

  setDisabled(disabled) { // client-only
    return new DashboardModel({ ...this, disabled });
  }
}
