import { AgGridReact } from 'ag-grid-react';
import debounce from 'lodash/debounce';
import * as PropTypes from 'prop-types';
import * as React from 'react';
import Spinner from '../../components/Spinner';
import ColumnsModel from './ColumnsModel';
import { LOAD_LIMIT } from './GridModel';
import HeaderSelection from './HeaderSelection';
import gridOptions from './options';

// https://material-ui.com/components/tables/
// https://www.ag-grid.com/javascript-grid-themes-provided/
// https://bluefletch.com/blog/choosing-a-data-grid-component-for-your-react-project/

const GRID_SELECTOR = '.ag-body-viewport';

class Grid extends React.Component<any> {

  public static propTypes() {
    return ({
      onLoad: PropTypes.func,
      columns: PropTypes.array,
      gridData: PropTypes.object.isRequired,
      onSaveColumnState: PropTypes.func,
      onRowSelected: PropTypes.func,
      columnState: PropTypes.object
    })
  }


  columnApi: any
  gridApi: any

  constructor(props) {
    super(props);
    this.onBodyScroll = debounce(this.onBodyScroll, 500).bind(this);
  }

  componentDidMount() {
    const { gridData } = this.props;
    if (gridData.isEmpty()) {
      this.loadGridData(gridData.getStart(), gridData.getStart() + LOAD_LIMIT);
    }
  }

  componentDidUpdate = (prevProps) => {
    const { gridData, columnState: newColumnState, columns } = this.props;
    const prevSortInfo = prevProps.gridData.getSortInfo();
    if (!prevProps.columns && this.gridApi) {
      // for when columns are loaded dynamically
      this.gridApi.setColumnDefs(columns);
    }
    if (gridData.isEmpty() && !gridData.isLoading()) {
      this.loadGridData(gridData.getStart(), gridData.getStart() + LOAD_LIMIT);
    } else if (prevSortInfo && !prevSortInfo.isEqual(gridData.getSortInfo())) {
      this.loadGridData(0, LOAD_LIMIT);
    }

    if (newColumnState && this.columnApi) {
      const oldColumnState = new ColumnsModel(this.columnApi.getColumnState());
      if (!oldColumnState.isEqual(newColumnState)) {
        this.columnApi.setColumnState(newColumnState.merge(oldColumnState).serialize());
        this.saveColumns();
      }
      if (!newColumnState.getColumns().length) {
        // Saving column state for the first time to fill modal
        this.saveColumns();
      }
    }
    this.autoFitColumns();
  }

  loadGridData = (start, end, callback?: any) => {
    const { onLoad, gridData } = this.props;
    if (onLoad) {
      onLoad(start, end, gridData.getSortInfo(), () => {
        if (callback) {
          callback();
        }
        this.autoFitColumns();
      });
    }
  }

  onGridReady = (params) => {
    this.gridApi = params.api;
    this.columnApi = params.columnApi;
    this.columnApi.headerSelection = new HeaderSelection(this.columnApi, this.gridApi);
    this.autoFitColumns();
  }

  autoFitColumns = () => {
    if (this.gridApi) {
      this.gridApi.sizeColumnsToFit();
    }
  }

  onRowSelected = () => {
    const { onRowSelected } = this.props;
    if (onRowSelected && this.gridApi) {
      const selectedItems = this.gridApi.getSelectedRows();
      onRowSelected(selectedItems);
    }
  }

  onRowDataChanged = () => {
    const { gridData } = this.props;
    if (this.gridApi) {
      if (gridData.isLoading() || gridData.getRowDataLength() > 0) {
        this.gridApi.hideOverlay();
      } else {
        this.gridApi.showNoRowsOverlay();
      }
    }
  }

  onColumnMoved = (ev) => {
    switch (ev.type) {
      case 'dragStopped':
        if (this.columnApi) {
          this.columnApi.headerSelection.redraw();
          this.saveColumns();
        }
        break;
      default:
        break;
    }
  }

  saveColumns = () => {
    const { onSaveColumnState } = this.props;
    if (onSaveColumnState) {
      const columnState = new ColumnsModel(this.columnApi.getColumnState());
      onSaveColumnState(columnState);
    }
  }

  onBodyScroll = (evt) => {
    const gridTop = evt.top;
    const { gridData } = this.props;
    const start = gridData.getStart();
    const end = gridData.getEnd();
    if (gridTop === 0) {
      if (gridData.getStart() > 0) {
        this.loadGridData(
          Math.max(start - LOAD_LIMIT, 0),
          end - LOAD_LIMIT,
          () => this.gridApi.ensureIndexVisible(LOAD_LIMIT)
        );
      }
    } else {
      const gridElem: any = document.querySelector(GRID_SELECTOR);
      if (gridElem && gridTop >= (gridElem.scrollHeight - gridElem.offsetHeight)) {
        this.loadGridData(
          end,
          end + LOAD_LIMIT,
          () => {
            const newGridData = this.props.gridData;
            if (newGridData.getStart() !== start || newGridData.getEnd() !== end) {
              const prevRowIndex = this.gridApi.getDisplayedRowCount() - LOAD_LIMIT - 1;
              this.gridApi.ensureIndexVisible(prevRowIndex, 'bottom');
            }
          });
      }
    }
  }

  componentWillUnmount() {
    this.gridApi = null;
    this.columnApi = null;
  }

  render() {
    const { gridData, columns, fullWidthComponent, ...other } = this.props;
    const rowData = gridData.getRowData();
    const options = {
      ...gridOptions,
      columnDefs: columns,
      onDragStopped: this.onColumnMoved,
      doesDataFlower: () => fullWidthComponent,
      isFullWidthCell: (rowNode) => rowNode.flower,
      fullWidthCellRendererFramework: fullWidthComponent
    };

    return (
      <div className='new-grid-container'>
        {gridData.isLoading() &&
          <div className='loading'>
            <Spinner />
          </div>}
        <AgGridReact
          {...other}
          onGridReady={this.onGridReady}
          onBodyScroll={this.onBodyScroll}
          onRowSelected={this.onRowSelected}
          onRowDataChanged={this.onRowDataChanged}
          gridOptions={options}
          rowData={rowData} />
      </div>
    );
  }
}

export default Grid;
