import * as React from 'react';
import classNames from 'classnames';
import { generateID } from '../../helpers';
import {
  ORDER,
  createAscendingSortInfo,
  createDescendingSortInfo
} from './HeaderModel';
import { HEADER_SELECTED } from './HeaderSelection';

const LEFT_CLICK = 0;
const RIGHT_CLICK = 2;

class HeaderRenderer extends React.Component<any, any> {
  constructor(props) {
    super(props);
    this.sortChanged = this.sortChanged.bind(this);
    this.onMouseUp = this.onMouseUp.bind(this);
    this.state = {
      sortOrder: props.sortOrder || props.column.sort,
      selected: false
    };
  }

  componentDidMount() {
    const { column, columnApi } = this.props;
    column.addEventListener('sortChanged', this.sortChanged);
    const headerSelection = columnApi.headerSelection;
    if (headerSelection && headerSelection.isHeaderSelected(column)) {
      headerSelection.applyStyle(column.colId);
      this.setState({
        selected: true
      });
    }
  }

  componentWillUnmount() {
    const { column } = this.props;
    column.removeEventListener('sortChanged', this.sortChanged);
  }

  sortChanged({ column }) {
    const { onSort } = this.props;
    this.setState({
      sortOrder: column.getSort()
    });

    if (onSort && column.getSort()) {
      const sortInfo = column.getSort() === ORDER.DESC
        ? createDescendingSortInfo(column.colId)
        : createAscendingSortInfo(column.colId);
      onSort(sortInfo);
    }
  }

  onMouseUp(ev) {
    const { column, columnApi, allowSelection } = this.props;
    const headerSelection = columnApi.headerSelection;

    if (ev.button === LEFT_CLICK) {
      const sortOrder = ev.target.dataset.sort;

      if (sortOrder) {
        this.props.setSort(sortOrder);
        return;
      }

      if (allowSelection) {
        if (ev.ctrlKey) {
          headerSelection.toggleSelectHeader(column);
        }

        if (!ev.ctrlKey && !ev.shiftKey) {
          headerSelection.clearSelection();
          headerSelection.selectHeader(column);
        }

        if (ev.shiftKey) {
          headerSelection.selectHeadersInRange(column);
        }
      }
    } else if (allowSelection && ev.button === RIGHT_CLICK) {
      if (!headerSelection.isHeaderSelected(column)) {
        headerSelection.clearSelection();
        headerSelection.selectHeader(column);
      }
    }
  }

  render() {
    const { column, onMouseDown, onContextMenu, children } = this.props;
    const { sortOrder, selected } = this.state;
    const isActionColumn = column.colDef.type === 'actionsColumn';

    return (
      <div id={generateID('Header', column.colId)}
        className={classNames('ag-header-component', { [HEADER_SELECTED]: selected })}
        onMouseUp={this.onMouseUp}
        onMouseDown={onMouseDown}
        onContextMenu={onContextMenu}>
        <div className='header-label-container'>
          <span className='label'>{column.colDef.headerName}</span>
        </div>
        {!isActionColumn &&
          <div className={classNames('arrows-container', { active: sortOrder })}>
            <span data-sort={ORDER.ASC} className={classNames('arrow arrow-up', {
              'arrow-selected': sortOrder === ORDER.ASC
            })} />
            <span data-sort={ORDER.DESC} className={classNames('arrow arrow-down', {
              'arrow-selected': sortOrder === ORDER.DESC
            })} />
          </div>}
        {children}
      </div>
    );
  }
}

export default HeaderRenderer;
