import useForceUpdate from './useForceUpdate'
import { useEffect, useMemo, useRef } from 'react'
import createPromiseResolver from './createPromiseResolver'
import createFetchError from './createFetchError'

export type ReadBody = (response: Response) => Promise<any>

export interface FetchOptions {
  readBody: ReadBody
}

const defaultReadBody: ReadBody = (response: Response) => response.json()

const defaultOpts: FetchOptions = {
  readBody: defaultReadBody
}

// https://itnext.io/usefetch-react-custom-hook-for-fetch-api-with-suspense-and-concurrent-mode-in-mind-1d3ba9250e0
export const useFetch = (input, opts: FetchOptions = defaultOpts) => {
  const forceUpdate = useForceUpdate();
  const error = useRef(null);
  const loading = useRef(false);
  const data = useRef(null);
  const promiseResolver = useMemo(createPromiseResolver, [input, opts]);

  useEffect(() => {
    let finished = false;
    const abortController = new AbortController();
    (async () => {
      if (!input) return;
      // start fetching
      loading.current = true;
      forceUpdate(0);
      const onFinish = (e, d) => {
        if (!finished) {
          finished = true;
          error.current = e;
          data.current = d;
          loading.current = false;
        }
      };
      try {
        const { readBody = defaultReadBody, ...init } = opts;
        const response = await fetch(input, {
          ...init,
          signal: abortController.signal,
        });
        // check response
        if (response.ok) {
          const body = await readBody(response)
          onFinish(null, body);
        } else {
          onFinish(await createFetchError(response), null)
        }
      } catch (e) {
        onFinish(e, null);
      }
      // finish fetching
      promiseResolver.resolve()
    })();
    
    const cleanup = () => {
      if (!finished) {
        finished = true
        abortController.abort()
      }
      error.current = null
      loading.current = false
      data.current = null
    };
    return cleanup;
  }, [input, opts])

  if (loading.current) {
    throw promiseResolver.promise
  }
  return {
    error: error.current,
    data: data.current,
  }
}

export default useFetch