import * as ip from '../../../../client/js/state/IpAddress';

const MAC_WITH_CIDR = 'mac with CIDR';
const MAC_WITH_CIDR_INVALID = 'mac with CIDR invalid';
const IPV4 = 'IPv4';
const WILDCARD = 'wildcard';
const IPV6 = 'IPv6';
const MAC_WITH_LEADING_ZEROS = 'Mac With leading zeros';
const MAC_WITH_LEADING_ZEROS_IP6 = 'Mac ip6 With leading zeros';
const IPV6_MAC = 'ipv6 mac';

const VALID_ADDRESS_CHECKS = 'valid address checks';
const INVALID_ADDRESS_CHECKS = 'invalid address checks';

const IS_MAC_ADDRESS_IN_CIDR = 'isMacAddressInCidr';
const IS_NOT_MAC_ADDRESS_IN_CIDR = '!isMacAddressInCidr';

const MAC_ON_IP6 = 'Mac in ip field';
const LEGACY_MAC_ON_IP6 = 'Legacy Mac on Ip field';

const MAC_WITH_BIG_CIDR = 'mac with big CIDR';
const IPV4_WITH_BIG_CIDR = 'ipv4 with big CIDR';

describe('IpAddress full address tests', () =>
  [{
    $name: MAC_WITH_CIDR,
    $in: {
      str: '9A53.D2CC.AA11/32',
      valueType: 'mac'
    },
    $out: {
      canonical: '9A:53:D2:CC:AA:11',
      prefix: 96
    }
  }, {
    $name: MAC_WITH_CIDR_INVALID,
    $in: {
      str: '9A53.D2CC./32',
      valueType: 'mac'
    },
    $out: {
      canonical: '::'
    }
  }, {
    $name: IPV4,
    $in: {
      str: '223.25.23.215',
      valueType: 'ip'
    },
    $out: {
      canonical: '223.25.23.215',
      prefix: 128
    }
  }, {
    $name: WILDCARD,
    $in: {
      str: '223.*',
      valueType: 'ip'
    },
    $out: {
      canonical: '223.0.0.0',
      prefix: 104
    }
  }, {
    $name: IPV6,
    $in: {
      str: '2001:0db8:0000:0042:0000:8a2e:0370:7334',
      valueType: 'ip6'
    },
    $out: {
      canonical: '2001:DB8::42:0:8A2E:370:7334',
      prefix: 128
    }
  }, {
    $name: IPV6_MAC,
    $in: {
      str: '2001:0db8:0000:0042:0000:8a2e:0370:7334',
      valueType: 'mac'
    },
    $out: {
      canonical: '2001:DB8::42:0:8A2E:370:7334',
      prefix: 128
    }
  }, {
    $name: MAC_WITH_LEADING_ZEROS,
    $in: {
      str: '00:53:02:cc:aa:11',
      valueType: 'mac'
    },
    $out: {
      canonical: '00:53:02:CC:AA:11',
      prefix: 128
    }
  }, {
    $name: MAC_WITH_LEADING_ZEROS_IP6,
    $in: {
      str: 'fe80::253:02ff:fecc:aa11',
      valueType: 'mac'
    },
    $out: {
      canonical: '00:53:02:CC:AA:11',
      prefix: 128
    }
  }, {
    $name: MAC_ON_IP6,
    $in: {
      str: 'fe80::253:02ff:fecc:aa11',
      valueType: 'ip6'
    },
    $out: {
      canonical: '00:53:02:CC:AA:11',
      prefix: 128
    }
  }, {
    $name: LEGACY_MAC_ON_IP6,
    $in: {
      str: '00:53:02:cc:aa:11',
      valueType: 'ip6'
    },
    $out: {
      canonical: '00:53:02:CC:AA:11',
      prefix: 128
    }
  }, {
    $name: MAC_WITH_BIG_CIDR,
    $in: {
      str: '9A53.D2CC.AA11/66',
      valueType: 'mac'
    },
    $out: {
      canonical: '9A:53:D2:CC:AA:11',
      prefix: 128
    }
  }, {
    $name: IPV4_WITH_BIG_CIDR,
    $in: {
      str: '0:0:0:0:0:ffff:fd6:8a46/96',
      valueType: 'ip'
    },
    $out: {
      canonical: '15.214.138.70',
      prefix: 128
    }
  }].forEach(testCase => {
    const { str, valueType } = testCase.$in;
    const { canonical, prefix } = testCase.$out;
    it(`Should correctly convert to bitstring and back for ${testCase.$name}`, () =>
      expect(ip.ipStringFromBits(ip.ipBitsFromString(str, valueType))).toEqual(canonical)
    );
    it(`Should correctly convert to bitstring and back for ${testCase.$name}`, () =>
      expect(ip.getPrefixBits(str, valueType)).toEqual(prefix)
    );
  })
);

describe('Valid Addresses checks', () =>
  [{
    $name: VALID_ADDRESS_CHECKS,
    $in: {
      isRelaxedIpForm: ['192.3.3.3'],
      isIpAddressInCidr: ['223.25.23.215/24', '192.*', '192.*.*.*'],
      isRelaxedIp6OrMacForm: ['2001:DB8::42:0:8A2E:370:7334'],
      isValidIP4: ['192.0.0.1'],
      isValidIP6: ['2001:DB8::42:0:8A2E:370:7334', '::FFFF:12.155.166.101'],
      isMacAddress: ['9A:53:D2:CC:AA:11', 'fe80:0000:0000:0000:9853:D2ff:feCC:AA11'],
      isMacAddressInCidr: ['9A:53:D2:CC:AA:11/12', 'fe80:0000:0000:0000:9853:D2ff:feCC:AA11/8'],
      isIpAddressType: ['ip', 'ipv4', 'ipv6', 'ip6', 'mac'],
      getPrefixedBitsByType: ['ip', 'ipv4', 'mac']
    }
  }].forEach((testCase) => Object.keys(testCase.$in).forEach(method =>
    testCase.$in[method].forEach((text) =>
      it(`Should correctly apply ${method} with ${testCase.$name} for ${text}`, () =>
        expect(ip[method](text)).toBeTruthy()
      )
    )
  ))
);

describe('Invalid addresses check', () =>
  [{
    $name: INVALID_ADDRESS_CHECKS,
    $in: {
      isRelaxedIpForm: ['1a92.s3.3.3', '123'],
      isIpAddressInCidr: ['223.25.23.21/5/24', 'as 192.*', '300.*.*.*'],
      isRelaxedIp6OrMacForm: ['200111DB8:7334'],
      isValidIP4: ['192.0.0.', '...', '192?0?0?1'],
      isValidIP6: ['2001:DB8:::0:8A2E:370:7334'],
      isMacAddress: ['zz:53:D2:CC:AA:11', 'abc'],
      isIpAddressType: ['abc', 'ip4', '123'],
      getPrefixedBitsByType: ['ip6', 'acv']
    }
  }].forEach((testCase) => Object.keys(testCase.$in).forEach(method =>
    testCase.$in[method].forEach((text) =>
      it(`Should correctly apply ${method} with ${testCase.$name} for ${text}`, () =>
        expect(ip[method](text)).toBeFalsy()
      )
    )
  ))
);

describe('isMacAddressInCidr check', () =>
  [{
    $name: IS_MAC_ADDRESS_IN_CIDR,
    $in: {
      text: '9A53.D2CC.AA11/32',
      strict: '9A53.D2CC.AA11'
    },
    $out: true
  }, {
    $name: IS_NOT_MAC_ADDRESS_IN_CIDR,
    $in: {
      text: '9A53.D2CC.AA11/32/64',
      strict: '9A53.D2CC.AA11'
    },
    $out: false
  }].forEach(testCase => {
    const { text, strict } = testCase.$in;
    it(`Should correctly check if mac address is in CIDR for ${testCase.$name}`, () => {
      expect(ip.isMacAddressInCidr(text, strict)).toBe(testCase.$out);
    });
  })
);
