import { ChartDisplay } from './state/chartDisplay'
import { Filter } from './state/filter'
import { createTimeline } from './state/timeline'
import { getAllFields, findTimeField } from './state/field'
import { getDefaultFieldset } from './state/fieldset'
import initialState from './state/initialState'
import { createSearchFieldset } from './state/searchFieldset'
import { SeSearch } from './state/seSearch'
import { Widget } from './state/widget'
import { ipStringFromBits, isIpAddressType, getPrefixedBitsByType } from './state/IpAddress'

const getField = (field) => getAllFields().find(fieldRef => (fieldRef.getField() === field))

const isDefaultTimeField = (key, value, timeField) => {
  if (key === timeField.field) {
    if (timeField.startDate === 0 && value.$lte === timeField.endDate && !value.$gte) {
      return true
    }
    return timeField.startDate === value.$gte && timeField.endDate === value.$lte
  }
  return false
}

const getSubnetSize = (subnetSize, fieldRef) => {
  const prefixedBits = fieldRef ? getPrefixedBitsByType(fieldRef.getValueType()) : 0
  return subnetSize - prefixedBits
}

class ConstructGenerator {
  constructor(query, generatorFunction, timeField, fieldRef) {
    this.query = query
    this.generatorFunction = generatorFunction
    this.timeField = timeField
    this.fieldRef = fieldRef
    this.keys = this.query ? Object.keys(this.query) : []
  }

  getInBetweenConstruct() {
    const { keys, query } = this
    if (keys.length === 2 && query.$gte && query.$lte) {
      return ` in between ${query.$gte} ${LOGIC_OPERATOR.$and} ${query.$lte}`
    }
    return undefined
  }

  getInSubnetConstruct() {
    const { keys, query, fieldRef } = this
    if (query.$or && keys.length === 1 && Array.isArray(query.$or)) {
      if (query.$or.every(q => !!q.$ipCidr)) {
        return `${SUBNET_OPERATOR.$ipCidr} ${query.$or.map(q =>
        `${ipStringFromBits(q.$ipCidr.ip)}${SLASH}${getSubnetSize(
          q.$ipCidr.subnetSize, fieldRef)}`)}`
      }
    } else if (query.$and && keys.length === 1 && Array.isArray(query.$and)) {
      if (query.$and.every(q => !!q.$ipCidrne)) {
        return `${SUBNET_OPERATOR.$ipCidrne} ${query.$and.map(q =>
          `${ipStringFromBits(q.$ipCidr.ip)}${SLASH}${getSubnetSize(
            q.$ipCidr.subnetSize, fieldRef)}`)}`
      }
    }
    return undefined
  }

  getGroupConstruct() {
    const { query, keys } = this
    const logicOperator = Object.keys(LOGIC_OPERATOR).find(key => !!query[key])
    if (logicOperator &&
      keys.length === 1 &&
      Array.isArray(query[logicOperator])) {
      const queryObject = query[logicOperator]
      const operator = Object.keys(SPECIAL_OPERATOR).find(key => !!queryObject[0][key])
      if (queryObject.every(q => !!q[operator])) {
        return `${SPECIAL_OPERATOR[operator]} ${queryObject.map(q => q[operator]).join(COMMA)}`
      }
    }
    return undefined
  }

  getNotInConstruct() {
    const { query, generatorFunction, timeField, fieldRef, keys } = this
    if (query.$or && keys.length === 1 && Array.isArray(query.$or)) {
      if (query.$or.length === 2) {
        const keys1 = Object.keys(query.$or[0])
        const keys2 = Object.keys(query.$or[1])
        if (keys1.length === 1 && keys2.length === 1 && keys1[0] === keys2[0]) {
          const kname = keys1[0]
          if (query.$or[0][kname].$null && query.$or[1][kname].$nin) {
            return `${kname} ${GROUP_OPERATOR.$nin} ${query.$or[1][kname]
              .$nin.map(e => generatorFunction(e, timeField, EMPTY, fieldRef)).join(COMMA)}`
          } else if (query.$or[1][kname].$null && query.$or[0][kname].$nin) {
            return `${kname} ${GROUP_OPERATOR.$nin} ${query.$or[0][kname]
              .$nin.map(e => generatorFunction(e, timeField, EMPTY, fieldRef)).join(COMMA)}`
          }
        }
      }
    }
    return undefined
  }

  getConstruct() {
    return this.getInBetweenConstruct() ||
      this.getInSubnetConstruct() ||
      this.getGroupConstruct() ||
      this.getNotInConstruct()
  }
}

const isBitString = (text) => /^[0-1]*$/.test(text)

const generateQuery = (query, timeField, joinOperator, fieldRef) => {
  if (query !== null && query !== undefined) {
    if (typeof query === 'object') {
      if (Array.isArray(query)) {
        return query
          .map(subQuery => generateQuery(subQuery, timeField, EMPTY, fieldRef))
          .filter(Boolean)
          .join(` ${joinOperator} `)
      }

      const construct = new ConstructGenerator(query, generateQuery, timeField, fieldRef)
        .getConstruct()
      if (construct) {
        return construct
      }

      const readableQuery = []
      Object.keys(query).forEach((key) => {
        if (key.indexOf('$') === 0) {
          if (LOGIC_OPERATOR[key]) {
            readableQuery.push(generateQuery(query[key], timeField, LOGIC_OPERATOR[key], fieldRef))
          } else if (UNARY_OPERATOR[key]) {
            readableQuery.push(`${query[key] ? 'is' : 'is not'} ${UNARY_OPERATOR[key]}`)
          } else if (OPERATOR[key]) {
            readableQuery
              .push(`${OPERATOR[key]} ${generateQuery(query[key], timeField, EMPTY, fieldRef)}`)
          } else if (GROUP_OPERATOR[key]) {
            readableQuery
              .push(`${GROUP_OPERATOR[key]} ${generateQuery(
                query[key], timeField, COMMA, fieldRef)}`)
          } else if (BINARY_OPERATOR[key]) {
            readableQuery.push(`${generateQuery(query[key], timeField, joinOperator, fieldRef)}`)
          } else if (SUBNET_OPERATOR[key]) {
            readableQuery
              .push(`${SUBNET_OPERATOR[key]} ${ipStringFromBits(
                query[key].ip)}${SLASH}${getSubnetSize(query[key].subnetSize, fieldRef)}`)
          }
        } else if (!isDefaultTimeField(key, query[key], timeField)) {
          const newFieldRef = getField(key)
          readableQuery
            .push(`${newFieldRef.getLabel()} ${generateQuery(
              query[key], timeField, OPERATOR.$eq, newFieldRef)}`)
        }
      })
      return readableQuery.join(` ${joinOperator} `)
    }
    const newJoin = joinOperator ? joinOperator.concat(' ') : EMPTY
    let quotes
    if (fieldRef) {
      quotes = fieldRef.getCefType() === 'string' ? '"' : EMPTY
      if (isIpAddressType(fieldRef.getValueType()) && !isBitString(query)) {
        return `${newJoin}${quotes}${query}${quotes}`
      }
      return `${newJoin}${quotes}${fieldRef.print(query)}${quotes}`
    }
    quotes = typeof query === 'string' ? '"' : EMPTY
    return `${newJoin}${quotes}${query}${quotes}`
  }
  return EMPTY
}

const getTimeField = (datePicker) => {
  const field = findTimeField().getField()
  return { field, startDate: datePicker.getStartDate(), endDate: datePicker.getEndDate() }
}

export const createQuery = (queryObject, datePicker, joinOperator) => {
  if (queryObject.query) {
    let newJoinOperator = joinOperator
    if (!newJoinOperator) {
      newJoinOperator = queryObject.query.$and || queryObject.query.$or
        ? OPERATOR.$eq
        : LOGIC_OPERATOR.$and
    }
    return generateQuery(queryObject.query, getTimeField(datePicker), newJoinOperator)
  }
  return EMPTY
}

export const createSearchFromWidget = (sourceWidget) => {
  const widget = new Widget(sourceWidget)
  const { filter, seSearch, queryObject } = widget.getSearch()
  const newSeSearch = new SeSearch(seSearch)
  const visualize = new ChartDisplay({}).addNewChart(widget.getContent().clone())
  const defaultFieldset = getDefaultFieldset()
  const fields = newSeSearch.getFields() || defaultFieldset.getFields()
  const fieldset = createSearchFieldset(defaultFieldset, fields)
  const datePicker = widget.getDatePicker()

  return {
    ...initialState.activeSearch,
    fieldset: fieldset.serialize(),
    name: widget.getName(),
    datePicker: datePicker.serialize(),
    timeline: createTimeline().adjustTo(datePicker).serialize(),
    query: widget.getQuery() || createQuery(queryObject, datePicker),
    filter: new Filter(filter).serialize(),
    visualize: visualize.serialize(),
    visualizations: visualize.getChartCount(),
    queryObject
  }
}
