import ipaddr from 'ipaddr.js';

const IP_TYPES = {
  IP: 'ip',
  IPV4: 'ipv4',
  IPV6: 'ipv6',
  IP6: 'ip6',
  MAC: 'mac'
};
const { IP, IPV4, IPV6, IP6, MAC } = IP_TYPES;
const INVALID = 'invalid';

const FF = 'FF';
const FE = 'FE';
const LINK_LOCAL_PREFIX = 'FE80:';

const BYTE_SIZE = 8;
const HEXADECIMAL = 16;
// const MAPPED_ADDRESS_FILLER_BITS = { [IPV4]: 96, [MAC]: 64 };

const WILDCARD = '*';
const COLON = ':';
const DOT = '.';
const SLASH = '/';

const isIpAddressType = (text) => [IP, IPV4, IPV6, IP6, MAC].includes(text);

// Look for values like: [12]3.[12]3.[12]3, [12]3.[12]3.[12]3.[12]3.[12]3.
const isRelaxedIpForm = (text) => /^(\d+\.){2,}(\d|\.)*$/.test(text);

// Looks for: n.*, n.n.*, n.n.n.*, n.*.*.*
const isIpWithWildCards = (text) => /^(\d+){1}(\.(\d+|\*)){0,2}(\.\*)$/.test(text);

const isRelaxedIp6OrMacForm = (text) =>
  /^(([0-9a-f]*:){3,}[0-9a-f]*|([0-9a-f]*:){2,}[0-9a-f]*(\.\d+)*)$/i.test(text);

/**
 * Determines whether the provided string correspond to a mac address eui48
 * Valid formats are: aa:aa:aa:aa:aa:aa, aa-aa-aa-aa-aa-aa and  aaaa.aaaa.aaaa
 * @param {byte array} A mac address in a form of a byte array of 16 octecs.
 * @return {boolean}
 */
const isValidLegacyMac = text =>
  /^[0-9a-f]{1,2}((:[0-9a-f]{1,2}){5}|(-[0-9a-f]{1,2}){5}|[0-9a-f]{2}(\.[0-9a-f]{4}){2})$/i.test(text); //eslint-disable-line

  // Looks for fe80:0000:0000:0000:aaaa:aaff:feaa:aaaa and FE80::aaaa:aaff:feaa:aaaa
const isContainedMacString = (text) =>
  /^(fe80(((:0){3}|(:0{4}){3})|:))(:[0-9a-f]{1,4}){0,1}(:[0-9a-f]{0,2}ff:fe[0-9a-f]{0,2}:[0-9a-f]{1,4})$/i.test(text) //eslint-disable-line

const isMacAddress = (text) => isValidLegacyMac(text) || isContainedMacString(text) ||
  /^(([0-9a-f]{4,4}:){3,3}|([0-9a-f]{4,4}-){3,3})[0-9a-f]{4,4}$/i.test(text);

/**
 * Determines whether the provided byte array correspond to a mac address with the link local prefix
 * @param {byte array} A mac address in a form of a byte array of 16 octecs.
 * @return {boolean}
 */
const isContainedMac = (byteArray) =>
  (byteArray[0] === 254 && byteArray[1] === 128 &&
  ![2, 3, 4, 5, 6, 7].some(index => byteArray[index] !== 0));

const isValidIP4 = (str) => {
  const split = str.split('.');
  return split.length === 4 && split.every(part => {
    if (part) {
      const p = parseInt(part, 10);
      return p >= 0 && p < 256;
    }
    return false;
  });
};

const isValidIP6 = (str) => {
  const split = str.split(':');
  let emptyPart = false; // if we have a "::" then we won't need exactly 8 parts
  const partsAreValid = split.length <= 8 && split.every((part, i) => {
    if (part) {
      if (i === split.length - 1 && isValidIP4(part)) {
        return true;
      }
      if (part.split('.').length > 1) { // do not allow dots in no IP4 parts
        return false;
      }
      const p = parseInt(part, 16);
      return p >= 0 && p <= 65535;
    }
    if (i > 0 && i < split.length - 1) {
      if (emptyPart) { // two empty parts are not allowed
        return false;
      }
      emptyPart = true;
    }
    return true;
  });
  return partsAreValid && (emptyPart || split.length === 8);
};

const allDelimiters = new RegExp(`[-${COLON}\\${DOT}]`);

const getDelimiter = (text) => {
  const matches = text.match(allDelimiters);
  return matches ? matches[0] : undefined;
};

const getPrefixedBitsByType = (type) => {
  switch (type) {
    case IPV4:
    case IP:
      return 96;
    case MAC:
      return 64;
    default:
      return 0;
  }
};

/**
 * Fills a string with zeros on the left side in order to complete the desired size.
 * @param {string} text
 * @param {int} size The target size for the binary string.
 * @returns {string} Returns a byte string (8 bits long)
 */
const addLeadingZeros = (text, size) => {
  const pad = size - text.length;
  return text && pad ? '0'.repeat(pad).concat(text) : text;
};

/**
 * Converts a decimal number to a 8 bit string
 * @param {decimal} decimal
 */
const toCompleteByteString = (decimal) => addLeadingZeros(decimal.toString(2), BYTE_SIZE);

/**
 * Inverts the U bit (seventh bit) of a given hexadecimal octec.
 * @param {hexadecimal} octec An hexadecimal octec (8 bits long)
 * @returns {hexadecimal} Returns the hexadecimal octec but with its 7th bit inverted.
 */
const toggleUBit = (octec) => {
  const bitArray = Array.from(toCompleteByteString(parseInt(octec, HEXADECIMAL)));
  const uBitIndex = bitArray.length - 2;
  bitArray[uBitIndex] = 1 - bitArray[uBitIndex];
  return parseInt(bitArray.join(''), 2).toString(HEXADECIMAL);
};

const toSixOctecsMac = (legacyMac, delimiter, fullAddressSize) => {
  const replaceRegex = new RegExp(`\\${delimiter}`, 'g');
  const fixedMac = legacyMac.length === (fullAddressSize - 1)
    ? '0'.concat(legacyMac)
    : legacyMac;
  return fixedMac.replace(replaceRegex, '').match(/.{2}/g);
};

const legacyMacToIpv6 = (legacyMac) => {
  const splitAddress = (legacyMac ? legacyMac.trim() : '').split(SLASH);
  if (splitAddress[0] && isValidLegacyMac(splitAddress[0])) {
    const delimiter = getDelimiter(splitAddress[0]);
    let eui64 = splitAddress[0].split(delimiter);
    switch (eui64.length) {
      case 4:
      case 6:
        break;
      case 3:
        // convert from [a]aaa.aaaa.aaaa
        eui64 = toSixOctecsMac(splitAddress[0], delimiter, 14);
        break;
      default:
        return splitAddress[0];
    }
    eui64.splice(3, 0, FF, FE);
    eui64[0] = toggleUBit(eui64[0]);

    return LINK_LOCAL_PREFIX.concat(eui64.reduce((acc, curr, idx) => (idx % 2 === 0
      ? acc.concat(COLON).concat(curr)
      : acc.concat(curr))
      , '')
    ).concat(splitAddress[1] ? `/${splitAddress[1]}` : '');
  }
  return legacyMac;
};

/**
 * Converts a contained Mac to eui48 format. In contained Mac, the first 8 bytes are ignored, from
 * the remaining bytes the 3rd and 4th are also ignored.
 * @param {array} byteArray Mac address in the form of a byte array.
 */
const containedMacArrayToEui48 = (byteArray) => {
  const newMac = byteArray.slice(8)
    .reduce((acc, curr, index) =>
      ((index !== 3 && index !== 4) ? acc.concat(curr.toString(HEXADECIMAL)) : acc), []);
  newMac[0] = toggleUBit(newMac[0]);
  return newMac.map(octec => (addLeadingZeros(octec.toString(), 2))).join(COLON);
};

const macIpv6ToLegacyMac = (ipAddress) => {
  const byteArray = ipAddress.toByteArray();
  return isContainedMac(byteArray)
    ? containedMacArrayToEui48(byteArray)
    : ipAddress.toString();
};

const isCidrBits = (text, max) => {
  const bits = parseInt(text, 10);
  return bits >= 0 && (!max || bits <= max);
};

const isMacAddressInCidr = (text, strict) => {
  const splitAddress = text.split(SLASH);
  if (splitAddress.length !== 2 || !isCidrBits(splitAddress[1], strict ? 64 : 0)) {
    return false;
  }

  return isMacAddress(splitAddress[0]);
};

/**
 * Converts a given IP using wildcards to a CIDR format. Supported formats are:
 * n.n.n.* converts to n.n.n.0/24
 * n.n.* or n.n.*.* converts to n.n.0.0/16
 * n.* or n.*.* or n.*.*.* converts to n.0.0.0/8
 * @param {string} ip The IP in any of the supported formats
 * @returns {string} Returns the IP in CIDR format
 */
const convertToCidrFormat = (ip) => {
  const IPBYTES = 4;
  if (isIpWithWildCards(ip)) {
    let maskBits = 0;
    const ipArray = ip.split(DOT).map((part, i) => {
      if (part === WILDCARD) {
        if (maskBits === 0 && i > 0) {
          maskBits = (IPBYTES - i) * BYTE_SIZE;
        }
        return '0';
      }
      return part;
    });
    while (ipArray.length < IPBYTES) {
      ipArray.push('0');
    }

    return ipArray.join(DOT).concat(SLASH).concat(32 - maskBits);
  }
  return ip;
};

const isIpAddressInCidr = (text, strict) => {
  const splitAddress = convertToCidrFormat(text).split(SLASH);
  if (splitAddress.length !== 2 || !isCidrBits(splitAddress[1], strict ? 128 : 0)) {
    return false;
  }

  return isValidIP4(splitAddress[0]) || isValidIP6(splitAddress[0]);
};

const parseCIDR = (text) => {
  const cidr = convertToCidrFormat(text);
  if (cidr && cidr.includes(SLASH)) {
    try {
      return ipaddr.parseCIDR(cidr);
    } catch (e) {
      return undefined;
    }
  }
  return undefined;
};

export default class IpAddress {
  constructor(ipAddress, cidrBits) {
    if (ipAddress && typeof ipAddress === 'object') {
      this.ipAddress = ipAddress.kind() !== IPV4 && ipAddress.isIPv4MappedAddress()
        ? ipAddress.toIPv4Address()
        : ipAddress;
      this.kind = isMacAddress(this.ipAddress.toNormalizedString()) ? MAC : this.ipAddress.kind();
      if (cidrBits < 0) {
        this.prefixBits = 128;
      } else {
        this.prefixBits = Math.min(cidrBits + getPrefixedBitsByType(this.kind), 128);
      }
    }
  }

  getPrefixBits() {
    return this.prefixBits;
  }

  getKind() {
    return this.kind || INVALID;
  }

  print() {
    const { ipAddress } = this;
    let printText = '';
    const kind = this.getKind();
    if (kind === MAC) {
      printText = macIpv6ToLegacyMac(this);
    } else {
      printText = ipAddress.toString();
    }
    return printText.toUpperCase();
  }

  toBitString() {
    return this.toByteArray().map(toCompleteByteString).join('');
  }

  toByteArray() {
    const ipAddress = this.ipAddress && this.ipAddress.kind() === IPV4
      ? this.ipAddress.toIPv4MappedAddress()
      : this.ipAddress;

    return ipAddress ? ipAddress.toByteArray() : [];
  }
}

const ipAddressFromString = (str, valueType) => {
  const ip = valueType === MAC || isMacAddress(str) ? legacyMacToIpv6(str) : str;
  const newIP = ip ? ip.trim() : ip;
  const cidr = parseCIDR(newIP);
  let ipAddress;
  let cidrBits;
  if (cidr) {
    ipAddress = cidr[0];
    cidrBits = cidr[1];
  } else {
    ipAddress = ipaddr.isValid(newIP)
      ? ipaddr.parse(newIP)
      : undefined;
    cidrBits = -1;
  }
  return new IpAddress(ipAddress, cidrBits);
};

const ipStringFromBits = (bits) => {
  const bytes = [];
  for (let i = 0; i < 128; i += 8) {
    const tmp = parseInt(bits.substring(i, i + 8), 2);
    bytes.push(tmp);
  }
  return new IpAddress(ipaddr.fromByteArray(bytes), -1).print();
};

const ipBitsFromString = (str, valueType) => ipAddressFromString(str, valueType).toBitString();
const getPrefixBits = (str, valueType) => ipAddressFromString(str, valueType).getPrefixBits();

exports.isValidLegacyMac = isValidLegacyMac;
exports.isRelaxedIpForm = isRelaxedIpForm;
exports.isRelaxedIp6OrMacForm = isRelaxedIp6OrMacForm;
exports.isMacAddress = isMacAddress;
exports.isValidIP4 = isValidIP4;
exports.isValidIP6 = isValidIP6;
exports.isMacAddressInCidr = isMacAddressInCidr;
exports.isIpAddressInCidr = isIpAddressInCidr;
exports.isIpAddressType = isIpAddressType;

exports.getPrefixBits = getPrefixBits;

exports.ipBitsFromString = ipBitsFromString;
exports.ipStringFromBits = ipStringFromBits;

exports.getPrefixedBitsByType = getPrefixedBitsByType;
