import moment from 'moment';
import { DatePicker } from '../../time/datePicker';
import { ipBitsFromString, ipStringFromBits } from './IpAddress';
import getGlobal from './refUtils';

const FULL_DATE_FORMAT = 'YYYY-MM-DD HH:mm:ss.SSS';

export function userInputAsValue(input, valueType) {
  switch (valueType) {
    case 'timestamp':
      return input ? moment(input).valueOf() || 0 : input;
    case 'ip':
    case 'ip6':
      return input ? ipBitsFromString(input) : input;
    case 'mac':
      return input ? ipBitsFromString(input, valueType) : input;
    case 'int':
    case 'float':
    case 'long':
    case 'port' :
    case 'uint' :
    case 'uid' :
    case 'short' :
    case 'processid' :
      if (input === null) {
        return null;
      }
      return input === '' || isNaN(+input) ? undefined : +input;
    default:
      return input;
  }
}

export function printValue(value, valueType, ...formattingArgs) {
  switch (valueType) {
    case 'timestamp':
      return DatePicker.formatTime(value, formattingArgs[0] || FULL_DATE_FORMAT);
    case 'outlier_anomalous_hour': {
      const timestamp = DatePicker.formatTime(value, formattingArgs[0] || FULL_DATE_FORMAT);
      // Converting the timestamp format to YYYY-MM-DD HH:mm-(HH+1):(mm+1)
      return timestamp.replace(/(\d\d):00:00\.000/, (_, hour) => {
        let nextConsecutiveHour = ((Number(hour) + 1) % 24)
          .toLocaleString(undefined, { mininumIntegerDigits: 2 });
        // If the next hour is single digit say 1 then it would become 01 (prefix padding)
        nextConsecutiveHour = String(nextConsecutiveHour).length === 1 ?
          '0'.concat(nextConsecutiveHour) : nextConsecutiveHour;
        return `${hour}:00-${nextConsecutiveHour}:00`;
      });
    }
    case 'mac':
    case 'ip':
    case 'ip6':
      return value ? ipStringFromBits(value) : value;
    case 'int':
      return value ? parseInt(value, 10) : value;
    case 'floatWith2Precision':
    default:
      return value;
  }
}

export class FieldRef {
  constructor(state, global) {
    this.id = state;
    this.global = global;
  }

  getId() {
    return this.id;
  }

  getLabel() {
    return this._getField().label;
  }

  // if available used for Job table headers in UI grid
  getDescription() {
    return this._getField().description;
  }

  // Do NOT access this field unless you are generating SQL
  getField() {
    return this.getName();
  }

  // Do NOT use outside lookup tables
  getFieldName() {
    return this._getField().name;
  }

  getName() {
    return this._getField().field || this._getField().name;
  }

  getGroupName() {
    return this._getField().groupName;
  }

  getSynonyms() {
    return this._getField().synonyms || [];
  }

  getValueType() {
    return this._getField().value_type;
  }

  getCefType() {
    const thisField = this._getField();
    const global = this.global || getGlobal();
    if (global && global.allFields && thisField.templateColumn) {
      const field = Object.values(global.allFields).find((elem) =>
        thisField.templateColumn === elem.field);
      return field ? field.cefType : null;
    }
    return thisField.cefType;
  }

  getRank() {
    return this._getField().rank;
  }

  isHidden() {
    return this._getField().isHidden;
  }

  isIdentity() {
    return this._getField().isIdentity;
  }

  isDefaultTimeFilter() {
    return this._getField().isDefaultTimeFilter;
  }

  isValid() {
    const global = this.global || getGlobal();
    return !!global && !!(
      (global.allFields && global.allFields[this.id])
      || (global.allLookupTableFields && global.allLookupTableFields[this.id])
    );
  }

  serialize() {
    return this.id;
  }

  isEqual(other) {
    return this.isValid() && this.id === other.id;
  }

  print(value, ...formattingArgs) {
    return printValue(value, this.getValueType(), ...formattingArgs);
  }

  userInputAsValue(userInput) {
    return userInputAsValue(userInput, this.getValueType());
  }

  _getField() {
    const global = this.global || getGlobal();
    if (global) {
      if (global.allFields && global.allFields[this.id]) {
        return global.allFields[this.id];
      }
      if (global.allLookupTableFields && global.allLookupTableFields[this.id]) {
        return global.allLookupTableFields[this.id];
      }
    }
    return {};
  }

  isLookupTableField() {
    // TODO: lookup fields should have tableId rather than tableName in them
    return !!this._getField().tableName;
  }
}

export const compareFields = (field1, field2) => {
  const label1 = field1.getLabel();
  const label2 = field2.getLabel();
  if (label1 < label2) {
    return -1;
  }
  if (label1 > label2) {
    return 1;
  }
  return 0;
};

export function getAllFields(global = getGlobal()) {
  const allFields = global ? Object.keys(global.allFields || {}).map(id =>
    new FieldRef(id, global)) : [];
  const allLookupTableFields = global ? Object.keys(global.allLookupTableFields || {}).map(id =>
    new FieldRef(id, global)) : [];
  return allFields.concat(allLookupTableFields);
}

export function findTimeField(global = getGlobal()) {
  return getAllFields(global).find(field => field.isDefaultTimeFilter());
}

export function findIdentityField(global = getGlobal()) {
  return getAllFields(global).find(field => field.isIdentity());
}

export const areFieldsValid = fieldIds =>
  fieldIds &&
  fieldIds.length > 0 &&
  fieldIds.every(id => new FieldRef(id).isValid());
