import moment from 'moment';

function _formatTime(milliseconds, format) {
  const dateTime = moment(milliseconds);
  return dateTime.isValid() ? dateTime.format(format) : '';
}

export const quickRanges = {
  LAST_1_MINUTE: 'Last 1 minute',
  LAST_5_MINUTES: 'Last 5 minutes',
  LAST_15_MINUTES: 'Last 15 minutes',
  LAST_30_MINUTES: 'Last 30 minutes',
  LAST_60_MINUTES: 'Last 60 minutes',
  LAST_6_HOURS: 'Last 6 hours',
  LAST_12_HOURS: 'Last 12 hours',
  LAST_24_HOURS: 'Last 24 hours',
  LAST_7_DAYS: 'Last 7 days',
  LAST_14_DAYS: 'Last 14 days',
  TODAY: 'Today',
  YESTERDAY: 'Yesterday',
  WEEK_TO_DATE: 'Week to date',
  PREVIOUS_WEEK: 'Last week',
  MONTH_TO_DATE: 'Month to date',
  PREVIOUS_MONTH: 'Last month',
  YEAR_TO_DATE: 'Year to date',
  ALL_TIME: 'All time'
};

const minute = 60;
const hour = 60 * minute;
const day = 24 * hour;
// TODO: quick ranges should not be declared with extra millisecond
const difference = {
  [quickRanges.LAST_1_MINUTE]: minute,
  [quickRanges.LAST_5_MINUTES]: 5 * minute,
  [quickRanges.LAST_15_MINUTES]: 15 * minute,
  [quickRanges.LAST_30_MINUTES]: 30 * minute,
  [quickRanges.LAST_60_MINUTES]: hour,
  [quickRanges.LAST_6_HOURS]: 6 * hour,
  [quickRanges.LAST_12_HOURS]: 12 * hour,
  [quickRanges.LAST_24_HOURS]: day,
  [quickRanges.LAST_7_DAYS]: 7 * day,
  [quickRanges.LAST_14_DAYS]: 14 * day
};

export const DATE_FORMAT = 'YYYY-MM-DD HH:mm:ss';

const CUSTOM_RANGE = 'Custom Range';

export class DatePicker {

  constructor({ startDate, endDate, datePickerLabel }) {
    this.startDate = startDate;
    this.endDate = endDate;
    this.datePickerLabel = datePickerLabel;
  }

  serialize() {
    const { startDate, endDate, datePickerLabel } = this;
    return { startDate, endDate, datePickerLabel };
  }

  static formatTime(milliseconds, dateFormat) {
    return _formatTime(milliseconds, dateFormat || DATE_FORMAT);
  }

  getStartDate() {
    return this.startDate;
  }

  getEndDate() {
    return this.endDate;
  }

  getLabel() {
    return this.datePickerLabel;
  }

  setLabel(datePickerLabel) {
    const newDatePicker = new DatePicker(this.serialize());
    newDatePicker.datePickerLabel = datePickerLabel;
    return newDatePicker;
  }

  setStartDate(startDate) {
    const newDatePicker = new DatePicker(this.serialize());
    newDatePicker.startDate = startDate;
    return newDatePicker;
  }

  setEndDate(endDate) {
    const newDatePicker = new DatePicker(this.serialize());
    newDatePicker.endDate = endDate;
    return newDatePicker;
  }

  printStartDate(format) {
    return DatePicker.formatTime(this.startDate, format);
  }

  printEndDate(format) {
    return DatePicker.formatTime(this.endDate, format);
  }

  isRelative() {
    return !!Object.values(quickRanges).includes(this.datePickerLabel);
  }

  isCustomRange() {
    return this.datePickerLabel === CUSTOM_RANGE;
  }

  setCustomRange() {
    return this.setLabel(CUSTOM_RANGE);
  }

  printInterval(dashStyle, format) {
    const { startDate, endDate } = this;
    if ((startDate && endDate) || dashStyle) {
      const prefix = dashStyle ? '' : 'From ';
      const infix = dashStyle ? ' - ' : ' until ';
      return `${prefix}${this.printStartDate(format)}${infix}${this.printEndDate(format)}`;
    }
    if (this.startDate) {
      return `From ${this.printStartDate(format)}`;
    }
    if (this.endDate) {
      return `Until ${this.printEndDate(format)}`;
    }
    return '';
  }

  difference(unit) {
    const startDate = this.getStartDate();
    const endDate = this.getEndDate();
    return unit ? moment(endDate).diff(startDate, unit) : endDate - startDate;
  }

  round(unit, inclusive) {
    const startDate = moment(this.getStartDate()).startOf(unit).valueOf();
    const endDate = moment(this.getEndDate()).endOf(unit).valueOf() + (inclusive && unit ? 1 : 0);
    if (startDate !== this.getStartDate() || endDate !== this.getEndDate()) {
      return this.setCustomRange().setStartDate(startDate).setEndDate(endDate);
    }
    return this;
  }

  generateGrid(timeline) {
    const unit = timeline.getGroupBy();
    const repeat = timeline.getGranularity();
    const from = moment(this.getStartDate()).startOf(unit);
    const to = moment(this.getEndDate()).endOf(unit);
    const grid = [];
    while (to.diff(from) > 0) {
      grid.push(from.valueOf());
      from.add(repeat, unit);
    }
    grid.push(to.valueOf());
    return grid;
  }

  refresh(newEndDate) {
    if (this.isCustomRange()) {
      return this;
    }

    const newDatePicker = new DatePicker(this.serialize());
    const { datePickerLabel } = this;
    let startDate = moment(newEndDate);
    const endDate = moment(newEndDate);
    switch (datePickerLabel) {
      case quickRanges.TODAY:
        startDate.startOf('day');
        break;
      case quickRanges.YEAR_TO_DATE:
        startDate.startOf('year');
        break;
      case quickRanges.MONTH_TO_DATE:
        startDate.startOf('month');
        break;
      case quickRanges.WEEK_TO_DATE:
        startDate.startOf('week');
        break;
      case quickRanges.YESTERDAY:
        startDate.subtract(1, 'days').startOf('day');
        endDate.subtract(1, 'days').endOf('day');
        break;
      case quickRanges.PREVIOUS_WEEK:
        startDate.subtract(1, 'weeks').startOf('week');
        endDate.subtract(1, 'weeks').endOf('week');
        break;
      case quickRanges.PREVIOUS_MONTH:
        startDate.subtract(1, 'months').startOf('month');
        endDate.subtract(1, 'months').endOf('month');
        break;
      case quickRanges.ALL_TIME:
        startDate = moment(0);
        break;
      default:
        startDate.subtract(difference[datePickerLabel], 'seconds');
        break;
    }
    newDatePicker.startDate = startDate.valueOf();
    newDatePicker.endDate = endDate.valueOf();
    return newDatePicker;
  }
}

function createDatePicker(startDate, endDate, datePickerLabel) {
  return new DatePicker({ startDate, endDate, datePickerLabel });
}

export function createCustomDatePicker(startDate, endDate) {
  return createDatePicker(startDate, endDate, CUSTOM_RANGE);
}

export function now() {
  return moment().valueOf();
}

function createDatePickerFromQuickRange(quickRange, time) {
  return createDatePicker(0, 0, quickRange).refresh(time);
}

export const quickDatePickers = Object
  .entries(quickRanges)
  .reduce((result, [key, value]) => ({
    ...result,
    [key]: createDatePickerFromQuickRange(value)
  }), {});

const formatToUnit = [
 { format: /^\d\d\d\d$/, unit: 'year' },
 { format: /^\d\d\d\d.\d?\d$/, unit: 'month' },
 { format: /^\d\d\d\d.\d?\d.\d?\d$/, unit: 'day' },
 { format: /^\d\d\d\d.\d?\d.\d?\d.\d?\d$/, unit: 'hour' },
 { format: /^\d\d\d\d.\d?\d.\d?\d.\d?\d.\d\d$/, unit: 'minute' },
 { format: /^\d\d\d\d.\d?\d.\d?\d.\d?\d.\d\d.\d\d$/, unit: 'second' }
];

export function getDateRange(datetimePrefix) {
  const date = moment(datetimePrefix);
  if (date.isValid()) {
    const startDate = moment(date).valueOf();
    const unitIndex = formatToUnit.findIndex(({ format }) => format.test(datetimePrefix));
    const endDate = unitIndex >= 0
      ? moment(date.valueOf() + 1).endOf(formatToUnit[unitIndex].unit).valueOf()
      : moment(date).valueOf();

    return createDatePicker(startDate, endDate);
  }
  return undefined;
}
