import * as moment from 'moment'
import { Timeline, TimeScale, TimeRange } from './types'
import { makeScale } from './utils/timeline'
import { timeDifference, translateTimeRangeLabel } from './utils'
import {
  SECONDS,
  MINUTES,
  HOURS,
  DAYS,
  MONTHS,
  YEARS
} from './constants/timeline'

import {
  LAST_1_MINUTE,
  LAST_5_MINUTES,
  LAST_15_MINUTES,
  LAST_30_MINUTES,
  LAST_60_MINUTES,
  LAST_6_HOURS,
  LAST_12_HOURS,
  LAST_24_HOURS,
  LAST_7_DAYS,
  LAST_14_DAYS,
  TODAY,
  YESTERDAY,
  WEEK_TO_DATE,
  PREVIOUS_WEEK,
  MONTH_TO_DATE,
  PREVIOUS_MONTH,
  YEAR_TO_DATE,
  ALL_TIME,
  CUSTOM_RANGE
} from './constants/time'
import { type } from 'os';

export const seconds: TimeScale[] = makeScale(31, SECONDS)
export const minutes: TimeScale[] = makeScale(31, MINUTES)
export const hours: TimeScale[] = makeScale(13, HOURS)
export const days: TimeScale[] = makeScale(15, DAYS)
export const months: TimeScale[] = makeScale(2, MONTHS)

export const minuteScales = [seconds[1], seconds[5], seconds[15]]
export const hourScales = [
  seconds[30],
  minutes[1],
  minutes[2],
  minutes[5],
  minutes[10],
  minutes[15],
  minutes[30],
  hours[1],
  hours[2],
  hours[4],
  hours[8]
]

export const dayScales = [
  hours[12],
  days[1],
  days[2],
  days[4],
  days[7],
  days[14]
]

const fixedScales = {
  [LAST_1_MINUTE]: seconds[1],
  [LAST_5_MINUTES]: seconds[5],
  [LAST_15_MINUTES]: seconds[15],
  [LAST_60_MINUTES]: minutes[1],
  [LAST_6_HOURS]: minutes[5],
  [LAST_12_HOURS]: minutes[10],
  [LAST_24_HOURS]: hours[1],
  [WEEK_TO_DATE]: hours[1],
  [TODAY]: hours[1],
  [YESTERDAY]: hours[1],
  [PREVIOUS_WEEK]: hours[1],
  [MONTH_TO_DATE]: days[1],
  [PREVIOUS_MONTH]: days[1],
  [YEAR_TO_DATE]: days[7],
  [ALL_TIME]: months[1]
}

export const getScale = (range: TimeRange): TimeScale => {
  type FitOptions = {
    scales: TimeScale[],
    groupBy: moment.unitOfTime.Diff,
    base: number,
    factor: number
  }
  const fitScale = ({
    scales,
    groupBy,
    base,
    factor
  }: FitOptions) => {
    const diff = timeDifference(range, groupBy);
    const reducer = ({ res, measure }, scale: TimeScale) => ({
      res: !res && diff <= measure ? scale : res,
      measure: measure * factor
    });
    return scales.reduce(reducer, { res: undefined, measure: base }).res;
  };

  let scale: TimeScale

  if (typeof range.fixed === 'string') {
    scale = fixedScales[range.fixed]
  }
  const fits: FitOptions[] = [
    { scales: minuteScales, groupBy: MINUTES, base: 3, factor: 3 },
    { scales: hourScales, groupBy: HOURS, base: 1, factor: 2 },
    { scales: dayScales, groupBy: DAYS, base: 70, factor: 2 }
  ]

  for (let i = 0; !scale && i < fits.length; i++) {
    scale = fitScale(fits[i])
  }

  return scale || months[1];
}

export const adjustTimelineRange = <T>(timeline: Timeline<T>, range: TimeRange): Timeline<T> => ({
  ...timeline,
  range,
  ...getScale(range)
})

export const createTimeline = <T>(range: TimeRange, collapsed: boolean, scale: TimeScale, data: T) => ({
  range,
  collapsed,
  ...scale,
  data
})