import { quickRanges } from './datePicker';
import { Range } from './range';
import { TimelineData } from './timelineData';

const minuteScales = [seconds[1], seconds[5], seconds[15]];
const hourScales = [
  seconds[30],
  minutes[1],
  minutes[2],
  minutes[5],
  minutes[10],
  minutes[15],
  minutes[30],
  hours[1],
  hours[2],
  hours[4],
  hours[8]
];
const dayScales = [
  hours[12],
  days[1],
  days[2],
  days[4],
  days[7],
  days[14]
];

function getScale(datePicker) {
  const fitScale = (scales, groupBy, base, factor) => {
    const diff = datePicker.difference(groupBy);
    const reductor = ({ res, measure }, scale) =>
      ({ res: !res && diff <= measure ? scale : res, measure: measure * factor });
    return scales.reduce(reductor, { res: undefined, measure: base }).res;
  };

  return {
    [quickRanges.LAST_1_MINUTE]: seconds[1],
    [quickRanges.LAST_5_MINUTES]: seconds[5],
    [quickRanges.LAST_15_MINUTES]: seconds[15],
    [quickRanges.LAST_60_MINUTES]: minutes[1],
    [quickRanges.LAST_6_HOURS]: minutes[5],
    [quickRanges.LAST_12_HOURS]: minutes[10],
    [quickRanges.LAST_24_HOURS]: hours[1],
    [quickRanges.WEEK_TO_DATE]: hours[1],
    [quickRanges.TODAY]: hours[1],
    [quickRanges.YESTERDAY]: hours[1],
    [quickRanges.PREVIOUS_WEEK]: hours[1],
    [quickRanges.MONTH_TO_DATE]: days[1],
    [quickRanges.PREVIOUS_MONTH]: days[1],
    [quickRanges.YEAR_TO_DATE]: days[7],
    [quickRanges.ALL_TIME]: months[1]
  }[datePicker.getLabel()]
    || fitScale(minuteScales, GROUP_BY.MINUTES, 3, 3)
    || fitScale(hourScales, GROUP_BY.HOURS, 1, 2)
    || fitScale(dayScales, GROUP_BY.DAYS, 70, 2)
    || months[1];
}

export const timelineFormats = {
  [GROUP_BY.SECONDS]: 'MMM DD YYYY, hh:mm:ssa',
  [GROUP_BY.MINUTES]: 'MMM DD YYYY, hh:mma',
  [GROUP_BY.HOURS]: 'MMM DD YYYY, hha',
  [GROUP_BY.DAYS]: 'MMM DD YYYY',
  [GROUP_BY.MONTHS]: 'MMM YYYY',
  [GROUP_BY.YEARS]: 'YYYY'
};

export class Timeline {
  constructor({ range, collapsed, granularity, groupBy, data }) {
    this.range = range;
    this.collapsed = collapsed;
    this.granularity = granularity;
    this.groupBy = groupBy;
    this.data = data;
  }

  serialize() {
    const { range, collapsed, granularity, groupBy, data } = this;
    return { range, collapsed, granularity, groupBy, data };
  }

  hasRange() {
    return !!this.range;
  }

  getRange() {
    return this.range ? new Range(this.range) : undefined;
  }

  isCollapsed() {
    return !!this.collapsed;
  }

  getGroupBy() {
    return this.groupBy;
  }

  getGranularity() {
    return this.granularity;
  }

  hasData() {
    return !!this.data;
  }

  getData() {
    return this.data ? new TimelineData(this.data) : undefined;
  }

  setData(data) {
    return new Timeline({
      ...this.serialize(),
      data: data ? data.serialize() : undefined
    });
  }

  setRange(range) {
    const newTimeline = new Timeline(this.serialize());
    newTimeline.range = range ? range.serialize() : undefined;
    return newTimeline;
  }

  setCollapsed(collapsed) {
    const newTimeline = new Timeline(this.serialize());
    newTimeline.collapsed = collapsed;
    return newTimeline;
  }

  setGroupBy(groupBy) {
    const newTimeline = new Timeline(this.serialize());
    newTimeline.groupBy = groupBy;
    return newTimeline;
  }

  setGranularity(granularity) {
    const newTimeline = new Timeline(this.serialize());
    newTimeline.granularity = granularity;
    return newTimeline;
  }

  getDateFormat() {
    return timelineFormats[this.getGroupBy()] || timelineFormats[GROUP_BY.SECONDS];
  }

  shrink() {
    return this.setData();
  }

  adjustTo(datePicker) {
    const scale = getScale(datePicker);
    return this.setGranularity(scale.getGranularity()).setGroupBy(scale.getGroupBy());
  }
}

export function createTimeline(rangeObj, collapsed, granularity, groupBy, dataObj) {
  const range = rangeObj ? rangeObj.serialize() : undefined;
  const data = dataObj ? dataObj.serialize() : undefined;
  return new Timeline({ range, collapsed, granularity, groupBy, data });
}
