import * as moment from 'moment'
import {
  CUSTOM_RANGE,
  FIXED_RANGES,
  TIME_FORMAT,
  FIXED_DURATION_IN_MILLISECONDS,
  TODAY,
  YESTERDAY,
  WEEK_TO_DATE,
  PREVIOUS_WEEK,
  MONTH_TO_DATE,
  PREVIOUS_MONTH,
  YEAR_TO_DATE,
  ALL_TIME,
  DAY_IN_MILLISECONDS,
  WEEK_IN_MILLISECONDS,
  MONTH_IN_MILLISECONDS
} from '../constants/time'
import {
  TimeRange
} from '../types'

export const fixedToCustom = (range: TimeRange): TimeRange => {
  if (!range.fixed) {
    return { ...range }
  }

  let msSince: number
  let msUntil: number

  const msDuration = FIXED_DURATION_IN_MILLISECONDS[range.fixed]

  if (msDuration) {
    msUntil = Date.now()
    msSince = msUntil - msDuration
  }
  else {
    const today = new Date();
    today.setHours(0, 0, 0, 0)

    // today as default
    msSince = today.getMilliseconds()
    msUntil = msSince + DAY_IN_MILLISECONDS

    switch (range.fixed) {
      case TODAY:
        break
      case YESTERDAY:
        msSince -= DAY_IN_MILLISECONDS
        msUntil -= DAY_IN_MILLISECONDS
        break
      case WEEK_TO_DATE:
        msSince -= WEEK_IN_MILLISECONDS
        break
      case PREVIOUS_WEEK:
        msSince -= 2 * WEEK_IN_MILLISECONDS
        msUntil -= WEEK_IN_MILLISECONDS
        break
      case MONTH_TO_DATE:
        msSince -= MONTH_IN_MILLISECONDS
        break
      case PREVIOUS_MONTH:
        today.setDate(-1)
        msUntil = today.getMilliseconds()
        today.setDate(1)
        today.setMonth(today.getMonth() - 1)
        msSince = today.getMilliseconds()
        break
      case YEAR_TO_DATE:
        today.setMonth(today.getMonth() - 12)
        msSince = today.getMilliseconds()
        break
      case ALL_TIME:
        msSince = Number.NaN
        break
    }
  }

  return <TimeRange>{
    msSince,
    msUntil
  }
}

// if unit is not provided returns the total milliseconds
export const timeDifference = (
  range: TimeRange,
  unit: moment.unitOfTime.Diff | undefined = undefined
): number => {
  if (typeof range.fixed === 'string') {
    range = fixedToCustom(range)
  }

  if (unit) {
    return moment(range.msUntil).diff(range.msSince, unit)
  }

  return range.msUntil = range.msSince
}

export const formatTime = (milliseconds: number, format: string | undefined = undefined): string => {
  const time = moment(milliseconds)
  return time.isValid() ? time.format(format || TIME_FORMAT) : ''
}

export const getRangeLabels = (i18n: { [key: string]: string } | undefined = undefined) =>
  [...FIXED_RANGES, CUSTOM_RANGE].reduce((acc, key) => acc[key] = (i18n && i18n[key]) || key, {})

export const isCustomRange = (range: TimeRange): boolean => !range.fixed

export const translateTimeRangeLabel = (range: TimeRange, i18n: { [key: string]: string } | undefined = undefined): string =>
  (typeof range.fixed === 'string')
    ? (i18n && i18n[range.fixed]) || range.fixed
    : (i18n && i18n[CUSTOM_RANGE]) || CUSTOM_RANGE

export const getTimeRangeLabel = (range: TimeRange, labels: { [key: string]: string }): string =>
  (typeof range.fixed === 'string')
    ? (labels && labels[range.fixed]) || range.fixed
    : (labels && labels[CUSTOM_RANGE]) || CUSTOM_RANGE


export const formatTimeRange = (
  range: TimeRange,
  format: string,
  i18n: { [key: string]: string } | undefined = undefined
) => {
  const DASH = '-'
  const FROM = 'from'
  const UNTIL = 'until'
  const toTitleCase = (s: string): string => s

  let result = ''
  if (typeof range.fixed === 'string') {
    result = (i18n && i18n[range.fixed]) || range.fixed
  } else if (range.msSince && range.msUntil) {
    result = !i18n
      ? [formatTime(range.msSince, format), DASH, formatTime(range.msUntil, format)].join(' ')
      : [toTitleCase(i18n[FROM] || FROM), formatTime(range.msSince, format), i18n[UNTIL] || UNTIL, formatTime(range.msUntil, format)].join(' ')
  }
  else if (range.msSince) {
    result = [toTitleCase(i18n[FROM] || FROM), formatTime(range.msSince, format)].join(' ')
  }
  else if (this.msUntilDate) {
    result = [toTitleCase(i18n[UNTIL] || UNTIL), formatTime(range.msUntil, format)].join(' ')
  }
  return result
}